const Mesa = require('vendor/Mesa.js');
const Shopify= require('vendor/Shopify.js');
const ShopifyGraphql = require('vendor/ShopifyGraphql.js');

/**
 * A Mesa Script exports a class with a script() method.
 */
module.exports = new class {

  /**
   * Mesa Script
   *
   * @param {object} payload The payload data
   * @param {object} context Additional context about this task
   */
  script = (payload, context) => {
    let vars = context.steps;

    let data = this.getFirstSkuWithInventory(vars);
    if (data) {
      Mesa.trigger.setTaskExternalData({
        label: "Free Gift: " + data.sku + ", available in warehouse: " + data.available,
      });  

      Mesa.output.next(data);
    }
  }

  getFirstSkuWithInventory = (vars) => {
    for (let sku in vars.transform) {
      let minimum = vars.transform[sku];
      Mesa.log.info("sku: " + sku + " - min: " + minimum);

      let data = this.getInventoryItemId(sku);
      let available = this.getInventoryLevel(vars, data.inventory_item_id);

      if (available >= minimum) {
        return {
          "sku": sku,
          "available": available,
          "inventory_item_id": data.inventory_item_id,
          "product_id": data.product_id,
          "variant_id": data.variant_id,
        };
      }
    }
  }

  getInventoryItemId = (sku) => {
    let query = `
      query($query: String!) {
        productVariants(first: 3, query: $query) {
          edges {
            node {
              id
              sku
              inventoryItem {
                id
              }
              product {
                id
              }
            }
          }
        }
      }
    `;

    const r = ShopifyGraphql.send(query, {
      "query": "sku:" + sku,
    });


    Mesa.log.info("response: ", r);

    let fullInventoryId = r.data.productVariants.edges[0].node.inventoryItem.id;
    let inventoryItemId = fullInventoryId.match(/\d+/)[0];

    let productGuid = r.data.productVariants.edges[0].node.product.id;
    let productId = productGuid.match(/\d+/)[0];

    let variantGuid = r.data.productVariants.edges[0].node.id;
    let variantId = variantGuid.match(/\d+/)[0];

    let data = {
      "inventory_item_id": inventoryItemId,
      "product_id": productId,
      "variant_id": variantId,
    }

    return data;
  }

  getInventoryLevel = (vars, itemId) => {
    let options = {
      query: {
        inventory_item_ids: itemId,
        location_ids: vars.transform_1.warehouse_id,
      }
    };
    let response = Shopify.get('/admin/inventory_levels.json', options);
    let available = response.inventory_levels[0].available;

    return available;
  }
}
